/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Mac OS X back-end
 */

/*
 * Most of the needed back-end functionality is actually provided by 
 * the cross-platform SDL back-end
 */


#include <sys/stat.h>
#include <CoreServices/CoreServices.h>

#include <SDL/SDL.h>


#define Uses_Game
#define Uses_Util
#define Uses_ProgramManager
#define Uses_DataFile
#include "mheader.h"


static const char *	ProgramDirectory[MAX_PROGRAM_DIRECTORIES] = {

	NULL,
	NULL,
	"config/",
	"graphics/",
	"terrain/",
	"objects/",
	"charact/",
	"text/",
	"world/"
	
};




/*
 * returns the path of a file
 */
char * get_file_path(char *path, PROGRAM_DIRECTORY directory,
    const char *file_name
) 
{
	FSRef fsref;
	CFBundleRef bundleref;
	struct stat s;

	switch(directory) {

	case DIR_USER_DATA:
		
		if (!FSFindFolder(kUserDomain, 
			kApplicationSupportFolderType, 
			kDontCreateFolder, &fsref)) {

			if (!FSRefMakePath( &fsref, (UInt8 *)path, 
				FILE_PATH_SIZE )) {

				/*
				 * Create save directory if it doesn't already exist.
                 * This part should probably go elsewhere so directories
                 * aren't created until the file is actually about to be
				 * opened. This should do for now, though.
				 */
				strcat(path, "/" GAME_NAME);

				if (stat(path, &s)) {
					
					mkdir(path, 0700);
				}

				if (!stat(path, &s)) {
					
					if (s.st_mode & S_IFDIR) {

						strcat(path, "/" );
						strcat(path, file_name);
						
						return path;
					}
				}
			}
		}

		die_no_data_path("*** BE_MAC ERROR *** invalid directory");

        return NULL;

	case DIR_DATA:

		if ((bundleref = CFBundleGetMainBundle()) != NULL) {
			CFURLRef url = CFBundleCopyBundleURL(bundleref);

			if (CFURLGetFileSystemRepresentation(url, true, 
				(UInt8 *)path, FILE_PATH_SIZE)) {
	
				strcat(path, "/Contents/Resources/");
				strcat(path, file_name);

				CFRelease(url);

				return path;
			}

			CFRelease(url);
		}
		
		die_no_data_path("*** BE_MAC ERROR *** invalid directory");

		return NULL;

	case DIR_CONFIG:
	case DIR_GRAPHICS:
	case DIR_TERRAIN:
	case DIR_OBJECTS:
	case DIR_CHARACTERS:
	case DIR_TEXT:
	case DIR_WORLD:
		
		if (get_file_path(path, DIR_DATA, 
			ProgramDirectory[directory])) {

			strcat(path, file_name);

			return path;
		}

		die_no_data_path("*** BE_MAC ERROR *** invalid directory");

		return NULL;

	default:

		die_no_data_path("*** BE_MAC ERROR *** invalid directory");
	
		return NULL;
	}
}



/*
 * main
 */
int main(int argc, char *argv[])
{
	NOT_USED(argc);
	NOT_USED(argv);

	run_game();

	return 0;
}


