/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Object
 * Description: -
 */


/*
 * misc. object related contants
 */
#define	MONEY_OBJECT_NAME			"Credit stick"


#define MIN_SG_FOR_TOME_READING			50



/*
 * object attribute constants
 */
#define	POWER_WEAPON_ARMOUR_VALUE_DIVIDER	2

#define	SHOCK_STUN_POWER			10

#define	FORCE_DAMAGE_BONUS			dice(1, 10)
#define	FORCE_RUNE_DAMAGE_BONUS			dice(2, 8)

#define	FORCE_VS_DAEMON_MULTIPLIER		2



/*
 * object attributes
 */
typedef enum {

	OA_NIL = -1,

	OA_UNIQUE,
	OA_DAEMONIC,
	OA_IMPASSABLE,
	OA_DANGEROUS,
	OA_BLOCKS_LINE_OF_SIGHT,
	OA_SHOCK,
	OA_PAIN,
	OA_POWER,
	OA_FORCE,
	OA_PROJECTILE,
	OA_LASER,
	OA_PLASMA,
	OA_PLAGUE,
	OA_POISON,
	OA_SILENT,
	OA_STRENGTH_BOOST,
	OA_STRENGTH_BOOST_2,
	OA_ENVIRONMENTAL_PROTECTION,
	OA_FORCE_RUNE,
	OA_FORCE_CHANNEL,
	OA_IGNORE_ARMOUR,
	OA_HINDERS_MOVEMENT,
	OA_PARTIAL_RELOAD,
	OA_AUTOMATIC_RECHARGE,
	OA_PSYCHIC_BOLT,
	OA_ALIEN,
	OA_RELIABLE,
	OA_NOISY,
	OA_INHERENT,
	OA_NO_PARRY,

	OA_OUT_OF_BOUNDS

} OBJECT_ATTRIBUTE;


/*
 * static object data
 */
typedef struct {

	/* common data ****************************/
	char			name[OBJECT_NAME_SIZE];

	bool			attribute[MAX_OBJECT_ATTRIBUTES];

	SYMBOL			symbol;
	DYNAMIC_COLOR		color;

	CONDITION		condition_max;

	CHARGE			charge_max;

	OBJECT_TYPE		type;
	OBJECT_SUBTYPE		subtype;

	WEIGHT			weight;

	MONEY			value;
	bool			buyable;
	/******************************************/

	/* weapons only ***************************/
	DICE_ROLL		damage;
	/******************************************/

	/* ranged combat weapons only *************/
	FIRING_MODE_DATA	firing_mode;
	RELOAD_VALUE		reload;
	/******************************************/

	/* close combat weapons only **************/
	WEAPON_REACH		reach;
	PARRY_PENALTY		parry_penalty;
	/******************************************/

	/* armours only ***************************/
	ARMOUR_VALUE		armour_value;
	/******************************************/

	/* source file ****************************/
	FILE_INDEX		source_file;
	/******************************************/

} OBJECT_DATA;


void			object_init(void);
void			object_clean_up(void);

OBJECT_DATA *		object_box_new_object(void);
void			object_box_optimize(void);

const OBJECT_DATA *	object_static_data(const OBJECT *);

bool			object_has_attribute(const OBJECT *,
				OBJECT_ATTRIBUTE
			);

OBJECT *		object_create(const char *);
OBJECT *		object_create_i(OBJECT_INDEX);

void			object_destroy(void *);

OBJECT *		object_clone(const OBJECT *);

void			object_screen(const OBJECT *);

char *			object_description(char *, const OBJECT *);

const char *		object_type_name(OBJECT_TYPE);
const char *		object_subtype_name(OBJECT_SUBTYPE);
const char *		object_attribute_name(OBJECT_ATTRIBUTE);

OBJECT_TYPE		name_to_object_type(const char *);
OBJECT_SUBTYPE		name_to_object_subtype(const char *);
OBJECT_ATTRIBUTE	name_to_object_attribute(const char *);

LIST *			object_list_trader(void);
LIST *			object_list_rogue_trader(void);
LIST *			object_list_drugs(CHARACTER *);
