#
# UNIX make file (only tested on Linux)
#


#
# IMPORTANT: this make file builds an i586 binary!
#
# you have to modify it if your target platform is not i586 compatible!
#


#
# IMPORTANT: the FTB builds do not work yet!
#



#
# Common source files
#
SRC_LIB=\
  lib/box.c \
  lib/cellpnt.c \
  lib/llist.c \
  lib/path.c \
  lib/rdb.c \
  lib/rng.c \
  lib/shake.c

SRC_UI=\
  ui/ui_input.c \
  ui/ui_cmd.c \
  ui/ui_c_bar.c \
  ui/ui_commo.c \
  ui/ui_game.c \
  ui/ui_g_adv.c \
  ui/ui_info.c \
  ui/ui_misc.c \
  ui/ui_msg.c \
  ui/ui_visio.c \
  ui/ui_menu.c \
  ui/ui.c \
  ui/ui_color.c \
  ui/ui_keys.c 

SRC_DUNGEON=\
  dungeon/d_hcity.c \
  dungeon/d_hwrld.c \
  dungeon/dungeon.c

SRC_MAIN=\
  actions.c \
  ai.c \
  area.c \
  areatran.c \
  career.c \
  charact.c \
  charadv.c \
  chargen.c \
  charstat.c \
  cmd.c \
  combat.c \
  combat_c.c \
  combat_r.c \
  datafile.c \
  death.c \
  debug.c \
  dice.c \
  drugs.c \
  dyn_msg.c \
  economy.c \
  effect.c \
  equip.c \
  event.c \
  faction.c \
  game.c \
  grammar.c \
  help.c \
  honour.c \
  inv.c \
  log.c \
  loadsave.c \
  macro.c \
  movement.c \
  npc.c \
  object.c \
  options.c \
  party.c \
  pathfind.c \
  percept.c \
  perks.c \
  prgman.c \
  psychic.c \
  randname.c \
  round.c \
  sector.c \
  talk.c \
  target.c \
  terrain.c \
  util.c \
  world.c



#
# goal: debug
#
ifeq ($(MAKECMDGOALS),debug)

SRC_BACK_END=\
  sdl/be_sdl.c \
  unix/be_unix.c

CFLAGS=-pipe -Wall -Wextra -ansi -pedantic -Wpointer-arith \
-Wstrict-prototypes -Wmissing-prototypes -Wdeclaration-after-statement \
-Wshadow -Wmissing-declarations -Wold-style-definition -Wredundant-decls \
-ggdb -D_FORTIFY_SOURCE=2 -DDEBUG -I. -I./lib -I./ui -I./dungeon `sdl-config --cflags`
LDFLAGS=-lm `sdl-config --libs`

STRIP_BINARY=no


#
# goal: release 
#
else ifeq ($(MAKECMDGOALS),release)

SRC_BACK_END=\
  sdl/be_sdl.c \
  unix/be_unix.c

CFLAGS=-std=c99 -Wall -march=i586 -Os -pipe -fomit-frame-pointer \
-I. -I./lib -I./ui -I./dungeon `sdl-config --cflags`
LDFLAGS=-Wl,-O1 -lm `sdl-config --libs`

STRIP_BINARY=yes


#
# goal: ftb_debug
#
else ifeq ($(MAKECMDGOALS),ftb_debug)

SRC_BACK_END=\
  curses/be_cur.c \
  unix/be_unix.c

CFLAGS=-pipe -Wall -Wextra -ansi -pedantic -Wpointer-arith \
-Wstrict-prototypes -Wmissing-prototypes -Wdeclaration-after-statement \
-Wshadow -Wmissing-declarations -Wold-style-definition -Wredundant-decls \
-ggdb -D_FORTIFY_SOURCE=2 -DDEBUG -DFOR_THE_BLIND -I. -I./lib -I./ui -I./dungeon
LDFLAGS=-lm -lncurses

STRIP_BINARY=no


#
# goal: ftb_release
#
else ifeq ($(MAKECMDGOALS),ftb_release)

SRC_BACK_END=\
  curses/be_cur.c \
  unix/be_unix.c

CFLAGS=-std=c99 -Wall -march=i586 -Os -pipe -fomit-frame-pointer \
-DFOR_THE_BLIND -I. -I./lib -I./ui -I./dungeon
LDFLAGS=-Wl,-O1 -lm -lncurses

STRIP_BINARY=yes


endif




SRC=\
  $(SRC_LIB) \
  $(SRC_BACK_END) \
  $(SRC_UI) \
  $(SRC_DUNGEON) \
  $(SRC_MAIN) 


OBJ=$(SRC:.c=.o)
EXE=./wrogue

CC=gcc
RM=rm -f


.PHONY : build
build: $(EXE)
	
	@echo Build complete!

ifeq ($(STRIP_BINARY),yes)
	@echo Stripping binary..
	@strip $(EXE)
endif

	@echo Copying files..
	@cp $(EXE) ..

ifeq ($(MAKECMDGOALS),release)
	@cp ./sdl/sdl.txt ..
	@cp ./unix/readme.txt ..
	@cp ./unix/runwrogue.sh ..
endif

	@echo Install complete!



.PHONY: debug
debug: build

.PHONY: release
release: build

.PHONY: ftb_release
ftb_release: build

.PHONY: ftb_debug
ftb_debug: build


.PHONY : clean
clean:
	-$(RM) *.o
	-$(RM) ./sdl/*.o
	-$(RM) ./curses/*.o
	-$(RM) ./lib/*.o
	-$(RM) ./ui/*.o
	-$(RM) ./dungeon/*.o
	-$(RM) ./unix/*.o
	-$(RM) $(EXE)

%.o: %.c
	@echo Building $<
	@$(CC) $(CFLAGS) -o $@ -c $<

$(EXE): $(OBJ)
	@$(CC) $(OBJ) $(LDFLAGS) -o $@

